model NoisepollutionGreenareas

/**
* Name: Effects of noise pollution on green areas
* Author: Yasmin Abdullayeva, Maija Saali
*/


global {
	//Shapefile of the buildings
	file business_buildings <- file("../includes/business.shp");
	file residential_buildings <- file("../includes/residential.shp");
	//Shapefile of the roads
	file roads <- file("../includes/eastroads.shp");
	//Shapefile of the green areas
	file greenareascas <- file("../includes/eastgreenareas.shp");
	//Shape of the environment
	geometry shape <- envelope(roads);
	//Step value
	float step <- 10 #s;
	//Graph of the road network
	graph road_network;
	//Map containing all the weights for the road network graph
	map<road,float> traffic_amount;
	
	init {
		//Initialization of the building using the shapefile of business buildings
		create business_building from: business_buildings;
		//Initialization of the building using the shapefile of residential buildings
		create residential_building from: residential_buildings;
		//Initialization of the road using the shapefile of roads
		create road from: roads;
		//Initialization of the green areas using shapefile of green areas
		create green_areas from: greenareascas;
		
		//Creation of the people agents
		create people number: 1000{
			//People agents are located anywhere in one of the building
			location <- any_location_in(one_of(residential_building));
      	}
      	//Weights of the road
      	traffic_amount <- road as_map (each::each.shape.perimeter);
     	road_network <- as_edge_graph(road);
	}
	
	//Reflex to update the speed of the roads according to the weights
	reflex update_road_speed  {
		traffic_amount <- road as_map (each::each.shape.perimeter / each.speed_coeff);
		road_network <- road_network with_weights traffic_amount;
	}
	
	//Reflex to decrease and diffuse the pollution of the environment
	reflex pollution_evolution{
		//ask all green_areas to decrease their level of noise pollution
		ask green_areas {noise_pollution <- noise_pollution * 0.7;}
	}
}

//Species to represent the people using the skill moving
species people skills: [moving]{
	//Target point of the agent
	point target;
	//Probability of leaving the building
	float leaving_proba <- 0.05; 
	
	//Probability of targeting business buildings
	float business_proba <- 0.50; 
	//Speed of the agent
	float speed <- 5 #km/#h;
	rgb color <- rnd_color(255);
	//Reflex to leave the building to another building
	reflex leave when: (target = nil) and (flip(leaving_proba)) {
		if (flip(business_proba)){
			target <- any_location_in(one_of(business_building));
		}
		else {
			target <- any_location_in(one_of(residential_building));
		}
	}
	//Reflex to move to the target building moving on the road network
	reflex move when: target != nil {
		//we use the return_path facet to return the path followed
		path path_followed <- goto (target: target, on: road_network, recompute_path: false, return_path: true, move_weights: traffic_amount);
		
		//if the path followed is not nil (i.e. the agent moved this step), we use it to increase the pollution level of overlapping cell
		if (path_followed != nil ) {
			ask (cell overlapping path_followed.shape) {
				// retrieving all overlapping agents at the current cell
				list<agent> temp_agents <- agents_overlapping(self);
				
				// defining variables that will be used at calculating noise pollution
				int cars_nearby <- 0;
				float total_speed_coeff <- 0.0;
				float avg_speed_coeff <- 0.0;
				float road_counter <- 0.0;
				
				// for each road we are getting speed_coeff variable, which indicates how fast cars are moving in the specific road
				ask(of_species(temp_agents, road)){
					total_speed_coeff <- total_speed_coeff + speed_coeff;
					road_counter <- road_counter + 1.0;
				}
				// after getting total speed coefficient, we are dividing it by road counter to get average speed coefficient of all roads that overlap the cell
				avg_speed_coeff <- total_speed_coeff / road_counter;
				
				// calculate number of cars (people) that are within the cell
				ask(of_species(temp_agents, people)){
					cars_nearby <- cars_nearby + 1;
				}

				// increase noise pollution of green areas that are within the cell
				ask(of_species(temp_agents, green_areas)){
					// if car is only one in the cell, add 15 (60 db is average for solo car in the highway, 60-45(default) = 15) multiplied by its speed
					if(cars_nearby = 1){
						noise_pollution <- noise_pollution + (15.0 * avg_speed_coeff);
					}
					// else add noise for each of additional cars nearby, multiplied by their speed
					else if(cars_nearby >= 1){
						noise_pollution <- noise_pollution + (5 * cars_nearby * avg_speed_coeff);
					}
				}
			}
		}
		if (location = target) {
			target <- nil;
		}	
	}
	aspect default {
		draw circle(5) color: color;
	}
}
//Species to represent the residential buildings
species residential_building {
	aspect default {
		draw shape color: #gray;
	}
}

//Species to represent the business buildings
species business_building {
	aspect default {
		draw shape color: #cyan;
	}
}

//Species to represent the roads
species road {
	//Capacity of the road considering its perimeter
	float capacity <- 1 + shape.perimeter/30;
	//Number of people on the road
	int nb_people <- 0 update: length(people at_distance 1);
	//Speed coefficient computed using the number of people on the road and the capicity of the road
	float speed_coeff <- 1.0 update:  exp(-nb_people/capacity) min: 0.1;
	int buffer<-3;
	aspect default {
		draw (shape + buffer * speed_coeff) color: #red;
	}
}

//Species to represent the green areas
species green_areas {
	float noise_pollution <- 45.0 min: 45.0 max: 120.0;
	
	//color updated according to the pollution level (from red - very polluted to green - no pollution)
	rgb color <- #green update: rgb(255 * ((noise_pollution-45.0)/30.0) , 255 * (1 - ((noise_pollution-45.0)/30.0)), 0.0);
}
	
//cell use to compute the pollution in the environment
grid cell height: 50 width: 50 neighbors: 8{
	rgb color <- #green;
}

experiment traffic type: gui {
	float minimum_cycle_duration <- 0.01;
	output {
		layout #split;
		display carte type: opengl{
			species green_areas;
			species residential_building refresh: false;
			species business_building refresh: false;
			species road ;
			species people ;
		}
		display Pie {
			chart "Noise pollution vs Green Areas" type: pie {
				data "Polluted" value: (green_areas as list) count (each.noise_pollution >= 75) color: #red;
				data "Clean" value: (green_areas as list) count (each.noise_pollution < 75) color: #green;
			} 
		}
		display Series {
			chart "Noise polltion vs Green Areas" type: series {
           		data "Polluted" value: (green_areas as list) count (each.noise_pollution >= 75) style: line color: #red;
				data "Clean" value: (green_areas as list) count (each.noise_pollution < 75) style: line color: #green;
        	 }
		}
	}
}
